from collections import OrderedDict

from .base import ApiBase
import logging

logger = logging.getLogger(__name__)


class Folders(ApiBase):
    def __init__(self, ns_client):
        ApiBase.__init__(self, ns_client=ns_client, type_name='Folder')

    def post(self, data) -> OrderedDict:
        assert data['externalId'], 'missing external id'
        folder = self.ns_client.Folder()

        if 'name' in data:
            folder['name'] = data['name']

        if 'externalId' in data:
            folder['externalId'] = data['externalId']

        logger.debug('able to create folder = %s', folder)
        res = self.ns_client.upsert(folder)
        return self._serialize(res)



from collections import OrderedDict

from .base import ApiBase
import logging

logger = logging.getLogger(__name__)


class Usage(ApiBase):
    SIMPLE_FIELDS = [
        'memo',
        'usageDate',
        'usageQuantity',
        'nullFieldList',
    ]

    RECORD_REF_FIELDS = [
        'customForm',
        'customer',
        'item',
        'subscriptionPlan',
        'usageSubscription',
        'usageSubscriptionLine',
    ]

    def __init__(self, ns_client):
        ApiBase.__init__(self, ns_client=ns_client, type_name='Usage')

    def post(self, data) -> OrderedDict:
        assert data['externalId'], 'missing external id'
        usage = self.ns_client.Usage(**data)

        self.build_simple_fields(self.SIMPLE_FIELDS, data, usage)

        self.build_record_ref_fields(self.RECORD_REF_FIELDS, data, usage)

        logger.debug('able to create usage = %s', usage)

        res = self.ns_client.upsert(usage)
        return self._serialize(res)



from collections import OrderedDict

from .base import ApiBase
import logging

logger = logging.getLogger(__name__)

class Terms(ApiBase):
    SIMPLE_FIELDS = [
        'dateDriven',
        'dayDiscountExpires',
        'dayOfMonthNetDue',
        'daysUntilExpiry',
        'daysUntilNetDue',
        'discountPercent',
        'discountPercentDateDriven',
        'dueNextMonthIfWithinDays',
        'externalId',
        'installment',
        'internalId',
        'isInactive',
        'percentagesList',
        'preferred',
        'recurrenceCount',
        'recurrenceFrequency',
        'repeatEvery',
        'splitEvenly',
        'nullFieldList',
    ]

    def __init__(self, ns_client):
        ApiBase.__init__(self, ns_client=ns_client, type_name='Term')

    def post(self, data) -> OrderedDict:
        term = self.ns_client.Term(daysUntilNetDue=data['daysUntilNetDue'], name=data['name'])

        self.build_simple_fields(self.SIMPLE_FIELDS, data, term)

        logger.debug('able to create term = %s', term)
        res = self.ns_client.request('add', record=term)
        return self._serialize(res)



from collections import OrderedDict

from .base import ApiBase
import logging

logger = logging.getLogger(__name__)

class Customers(ApiBase):
    SIMPLE_FIELDS = [
        'accountNumber',
        'addressbookList',
        'aging',
        'aging1',
        'aging2',
        'aging3',
        'aging4',
        'alcoholRecipientType',
        'altEmail',
        'altName',
        'altPhone',
        'balance',
        'billPay',
        'clickStream',
        'comments',
        'companyName',
        'consolAging',
        'consolAging1',
        'consolAging2',
        'consolAging3',
        'consolAging4',
        'consolBalance',
        'consolDaysOverdue',
        'consolDepositBalance',
        'consolOverdueBalance',
        'consolUnbilledOrders',
        'contactRolesList',
        'contribPct',
        'creditCardsList',
        'creditHoldOverride',
        'creditLimit',
        'currencyList',
        'customFieldList',
        'dateCreated',
        'daysOverdue',
        'defaultAddress',
        'defaultOrderPriority',
        'depositBalance',
        'displaySymbol',
        'downloadList',
        'email',
        'emailPreference',
        'emailTransactions',
        'endDate',
        'entityId',
        'estimatedBudget',
        'externalId',
        'fax',
        'faxTransactions',
        'firstName',
        'firstVisit',
        'giveAccess',
        'globalSubscriptionStatus',
        'groupPricingList',
        'homePhone',
        'internalId',
        'isBudgetApproved',
        'isInactive',
        'isPerson',
        'itemPricingList',
        'keywords',
        'language',
        'lastModifiedDate',
        'lastName',
        'lastPageVisited',
        'lastVisit',
        'middleName',
        'mobilePhone',
        'negativeNumberFormat',
        'numberFormat',
        'openingBalance',
        'openingBalanceDate',
        'overdueBalance',
        'overrideCurrencyFormat',
        'partnersList',
        'password',
        'password2',
        'phone',
        'phoneticName',
        'printOnCheckAs',
        'printTransactions',
        'referrer',
        'reminderDays',
        'requirePwdChange',
        'resaleNumber',
        'salesTeamList',
        'salutation',
        'sendEmail',
        'shipComplete',
        'stage',
        'startDate',
        'subscriptionsList',
        'symbolPlacement',
        'syncPartnerTeams',
        'taxExempt',
        'taxRegistrationList',
        'taxable',
        'thirdPartyAcct',
        'thirdPartyCountry',
        'thirdPartyZipcode',
        'title',
        'unbilledOrders',
        'url',
        'vatRegNumber',
        'visits',
        'webLead',
        'nullFieldList',
    ]

    RECORD_REF_FIELDS = [
        'accessRole',
        'assignedWebSite',
        'buyingReason',
        'buyingTimeFrame',
        'campaignCategory',
        'category',
        'customForm',
        'defaultTaxReg',
        'drAccount',
        'entityStatus',
        'fxAccount',
        'image',
        'leadSource',
        'openingBalanceAccount',
        'parent',
        'partner',
        'prefCCProcessor',
        'priceLevel',
        'receivablesAccount',
        'salesGroup',
        'salesReadiness',
        'salesRep',
        'shippingItem',
        'sourceWebSite',
        'taxItem',
        'terms',
        'territory',
    ]

    def __init__(self, ns_client):
        ApiBase.__init__(self, ns_client=ns_client, type_name='Customer')

    def post(self, data) -> OrderedDict:
        assert data['externalId'], 'missing external id'
        customer = self.ns_client.Customer(externalId=data['externalId'])

        customer['currency'] = self.ns_client.RecordRef(**(data['currency']))

        customer['subsidiary'] = self.ns_client.RecordRef(**(data['subsidiary']))

        customer['representingSubsidiary'] = self.ns_client.RecordRef(**(data['representingSubsidiary']))

        customer['monthlyClosing'] = self.ns_client.RecordRef(**(data['monthlyClosing']))

        self.build_simple_fields(self.SIMPLE_FIELDS, data, customer)

        self.build_record_ref_fields(self.RECORD_REF_FIELDS, data, customer)

        logger.debug('able to create customer = %s', customer)
        res = self.ns_client.upsert(customer)
        return self._serialize(res)



from collections import OrderedDict

from .base import ApiBase
import logging

logger = logging.getLogger(__name__)


class Files(ApiBase):
    def __init__(self, ns_client):
        ApiBase.__init__(self, ns_client=ns_client, type_name='File')

    def post(self, data) -> OrderedDict:
        assert data['externalId'], 'missing external id'
        file = self.ns_client.File()

        if 'name' in data:
            file['name'] = data['name']

        if 'folder' in data:
            file['folder'] = self.ns_client.RecordRef(**(data['folder']))

        if 'externalId' in data:
            file['externalId'] = data['externalId']

        if 'content' in data:
            file['content'] = data['content']

        if 'mediaType' in data:
            file['mediaType'] = data['mediaType']

        logger.debug('able to create file = %s', file)
        res = self.ns_client.upsert(file)
        return self._serialize(res)



from .api.accounts import Accounts
from .api.billing_account import BillingAccount
from .api.classifications import Classifications
from .api.credit_memos import CreditMemos
from .api.departments import Departments
from .api.currencies import Currencies
from .api.locations import Locations
from .api.vendor_bills import VendorBills
from .api.vendor_credits import VendorCredits
from .api.vendors import Vendors
from .api.subsidiaries import Subsidiaries
from .api.usage import Usage
from .api.journal_entries import JournalEntries
from .api.adv_inter_company_journal_entries import AdvInterCompanyJournalEntries
from .api.employees import Employees
from .api.expense_reports import ExpenseReports
from .api.folders import Folders
from .api.files import Files
from .api.customers import Customers
from .api.projects import Projects
from .api.expense_categories import ExpenseCategory
from .api.custom_lists import CustomLists
from .api.custom_segments import CustomSegments
from .api.custom_record_types import CustomRecordTypes
from .api.custom_records import CustomRecords
from .api.vendor_payments import VendorPayments
from .api.invoices import Invoices
from .api.terms import Terms
from .api.tax_items import TaxItems
from .api.tax_groups import TaxGroups
from .internal.client import NetSuiteClient


class NetSuiteConnection:
    def __init__(self, account, consumer_key, consumer_secret, token_key, token_secret,
                 caching=True, caching_timeout=2592000, caching_path=None,
                 search_body_fields_only=True, page_size: int = 100):
        ns_client = NetSuiteClient(account=account, caching=caching, caching_timeout=caching_timeout,
                                   caching_path=caching_path, search_body_fields_only=search_body_fields_only,
                                   page_size=page_size)
        ns_client.connect_tba(
            consumer_key=consumer_key,
            consumer_secret=consumer_secret,
            token_key=token_key,
            token_secret=token_secret
        )
        self.client = ns_client
        self.accounts = Accounts(ns_client)
        self.billing_accounts = BillingAccount(ns_client)
        self.classifications = Classifications(ns_client)
        self.departments = Departments(ns_client)
        self.currencies = Currencies(ns_client)
        self.locations = Locations(ns_client)
        self.vendor_bills = VendorBills(ns_client)
        self.vendor_credits = VendorCredits(ns_client)
        self.vendors = Vendors(ns_client)
        self.subsidiaries = Subsidiaries(ns_client)
        self.journal_entries = JournalEntries(ns_client)
        self.adv_inter_company_journal_entries = AdvInterCompanyJournalEntries(ns_client)
        self.employees = Employees(ns_client)
        self.expense_reports = ExpenseReports(ns_client)
        self.folders = Folders(ns_client)
        self.files = Files(ns_client)
        self.expense_categories = ExpenseCategory(ns_client)
        self.custom_lists = CustomLists(ns_client)
        self.custom_segments = CustomSegments(ns_client)
        self.custom_records = CustomRecords(ns_client)
        self.custom_record_types = CustomRecordTypes(ns_client)
        self.customers = Customers(ns_client)
        self.projects = Projects(ns_client)
        self.vendor_payments = VendorPayments(ns_client)
        self.invoices = Invoices(ns_client)
        self.terms = Terms(ns_client)
        self.tax_items = TaxItems(ns_client)
        self.tax_groups = TaxGroups(ns_client)
        self.credit_memos = CreditMemos(ns_client)
        self.usages = Usage(ns_client)



from collections import OrderedDict

from .base import ApiBase
import logging

logger = logging.getLogger(__name__)


class BillingAccount(ApiBase):
    SIMPLE_FIELDS = [
        'createdBy',
        'createdDate',
        'customFieldList',
        'customerDefault',
        'frequency',
        'idNumber',
        'inactive',
        'lastBillCycleDate',
        'lastBillDate',
        'memo',
        'name',
        'nextBillCycleDate',
        'startDate',
        'nullFieldList',
    ]

    RECORD_REF_FIELDS = [
        'billingSchedule',
        'cashSaleForm',
        'class',
        'currency',
        'customForm',
        'customer',
        'department',
        'invoiceForm',
        'location',
        'subsidiary',
    ]

    def __init__(self, ns_client):
        ApiBase.__init__(self, ns_client=ns_client, type_name='BillingAccount')

    def post(self, data) -> OrderedDict:
        assert data['externalId'], 'missing external id'
        billing_account = self.ns_client.BillingAccount(**data)

        self.build_simple_fields(self.SIMPLE_FIELDS, data, billing_account)

        self.build_record_ref_fields(self.RECORD_REF_FIELDS, data, billing_account)

        logger.debug('able to create billing account = %s', billing_account)

        res = self.ns_client.upsert(billing_account)
        return self._serialize(res)
